
# q-expansion of newform 4032.2.a.w, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4032
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [1], [0], [4], [-6], [-2], [0], [-6], [-4], [-2], [-6], [-8], [12], [6], [-6], [-8], [4], [0], [2], [8], [-6], [6], [-10], [0], [-4], [12], [-2], [-6], [-16], [18], [-18], [-14], [-18], [8], [4], [16], [12], [-12], [-12], [-20], [-24], [14], [-18], [20], [4], [8], [18], [4], [6], [-24], [-10], [-18], [-18], [0], [-12], [-16], [10], [6], [22], [24], [-2], [24], [-10], [6], [-8], [14], [-24], [28], [-18], [24], [8], [-14], [16], [-36], [18], [-20], [18], [14], [6], [10], [-24], [-34], [8], [-12], [-18], [-10], [12], [32], [-6], [36], [-16], [-12], [4], [0], [36], [-6], [-2], [-38], [-8], [6], [30], [-6], [-32], [2], [-42], [6], [24], [26], [32], [-2], [-6], [-26], [-16], [18], [-14], [12], [18], [-24], [40], [26], [-12], [-12], [46], [18], [46], [-12], [44], [40], [16], [-24], [-40], [-2], [18], [14], [24], [22], [-12], [-6], [-2], [6], [-40], [-36], [-56], [-12], [-44], [18], [-14], [24], [22], [54], [-20], [36], [-44], [-48], [56], [-6], [2], [-24], [24], [54], [32], [-6], [6], [36], [-16], [-8], [-34], [-36], [36], [4], [0], [26], [-4], [30], [-44], [-30], [-16], [-8], [8], [30], [22], [6], [-48], [36], [34], [46], [50], [12], [2], [-60], [-20], [60], [-12], [-66], [14], [-26], [6], [24], [30], [-28], [40], [26], [18], [24], [20], [0], [-30], [58], [-34], [-48], [8], [42], [24], [62], [32], [18], [24], [-30], [22], [-40], [-18], [56], [-30], [-26], [-42], [-24], [-64], [-48], [-50], [-38], [32], [6], [46], [0], [-58], [30], [-12], [0], [-72], [-2], [32], [34], [-42], [12], [-16], [66], [40], [0], [58], [-66], [-48], [26], [-72], [-60], [-74], [10], [0], [-34], [8], [48], [-32], [-56], [42], [10], [54], [6], [4], [-6], [76], [-56], [-46], [32], [50], [-16], [72], [10], [-34], [42], [24], [-40], [-12], [-8], [-50], [-48], [2], [78], [44], [-18], [-18], [42], [-54], [-30], [-62], [84], [-64], [-60], [30], [82], [14], [-78], [-16], [-48], [40], [-70], [-72], [-32], [0], [-38], [-24], [-30], [-42], [-32], [0], [-70], [30], [-48], [26], [30], [10], [50], [-72], [56], [-54], [-34], [88], [46], [48], [-54], [10], [-78], [-28], [54], [-92], [18], [-2], [-18], [-22], [20], [82], [42], [-12], [-64], [-30], [-72], [-20], [-80], [24], [-24], [-14], [-70], [-66], [-28], [-14], [-18], [36], [42], [-18], [-48], [-62], [30], [48], [60], [-2], [-10], [84], [-16], [38], [36], [34], [0], [30], [-40], [-38], [-78], [-72], [-70], [42], [-82], [36], [-6], [56], [-18], [-14], [-24], [8], [-32], [-20], [12], [62], [-48], [60], [10], [-24], [-94], [-52], [42], [4], [66], [40], [78], [32], [18], [60], [92], [-62], [-30], [-86], [-66], [26], [42], [0], [-44], [-22], [12], [24], [44], [-54], [0], [18], [-56], [96], [6], [38], [-96], [24], [54], [-38], [-36]]
